==> cpg_flow-0.1.0a11.tar.gz: cpg_flow-0.1.0a11/tests/stages/test_stage_types.py <==
"""
Test building stages DAG.
"""

from pytest_mock import MockFixture

from tests import set_config
from tests.stages import (
    CohortStage1,
    DatasetStage1,
    DatasetStage2,
    MultiCohortStage1,
    SGStage1,
    mock_multicohort,
    mock_multidataset_cohort,
    run_workflow,
)


def set_config_to_default(tmp_path):
    set_config(get_config(tmp_path=tmp_path), tmp_path / 'config.toml')


def print_job_by_stage():
    from cpg_utils.hail_batch import get_batch

    print('Job by stage:', get_batch().job_by_stage)


def get_config(tmp_path) -> str:
    conf = f"""
    [workflow]
    access_level = 'test'
    dataset = 'fewgenomes'
    sequencing_type = 'genome'
    driver_image = 'stub'

    check_inputs = false
    check_intermediates = false

    # Skip stages with outputs that already exist
    check_expected_outputs = false

    [storage.default]
    default = '{tmp_path}'
    [storage.fewgenomes]
    default = '{tmp_path}'

    [hail]
    billing_project = 'fewgenomes'
    backend = 'local'
    dry_run = true
    """

    return conf


def get_config_multicohort(tmp_path) -> str:
    conf = f"""
    [workflow]
    access_level = 'test'
    dataset = 'fewgenomes'
    sequencing_type = 'genome'
    driver_image = 'stub'
    input_cohorts = ['COHA', 'COHB']

    check_inputs = false
    check_intermediates = false

    # Skip stages with outputs that already exist
    check_expected_outputs = false

    [storage.default]
    default = '{tmp_path}'
    [storage.fewgenomes]
    default = '{tmp_path}'

    [hail]
    billing_project = 'fewgenomes'
    backend = 'local'
    dry_run = true
    """

    return conf


def test_dataset_stages(mocker: MockFixture, tmp_path):
    """
    Test that dataset stages are run once per dataset.
    """
    set_config_to_default(tmp_path)

    from cpg_flow.inputs import get_multicohort
    from cpg_utils.hail_batch import get_batch

    run_workflow(mocker, cohort_mocker=mock_multidataset_cohort, stages=[DatasetStage1])
    print_job_by_stage()
    assert get_batch().job_by_stage.get('DatasetStage1')
    # The next two tests are looking at the same behaviour, but from different angles
    assert len(get_multicohort().get_datasets()) == 2
    assert get_batch().job_by_stage.get('DatasetStage1').get('job_n') == len(get_multicohort().get_datasets())


def test_multiple_dataset_stages(mocker: MockFixture, tmp_path):
    """
    Test that dataset stages are run once per dataset.
    """

    set_config_to_default(tmp_path)

    from cpg_flow.inputs import get_multicohort
    from cpg_utils.hail_batch import get_batch

    run_workflow(mocker, cohort_mocker=mock_multidataset_cohort, stages=[DatasetStage1, DatasetStage2])
    print_job_by_stage()
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('DatasetStage2')

    assert len(get_multicohort().get_datasets()) == 2
    assert get_batch().job_by_stage.get('DatasetStage1').get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('DatasetStage2').get('job_n') == len(get_multicohort().get_datasets())


def test_mixture_dataset_stage_sg_stage(mocker: MockFixture, tmp_path):
    """
    Tests that when a SequencingGroupStage and DatasetStage are run together, the DatasetStage is run once per dataset and
    the SequencingGroupStage is run once per sequencing group.
    """

    set_config_to_default(tmp_path)

    from cpg_flow.inputs import get_multicohort
    from cpg_utils.hail_batch import get_batch

    run_workflow(mocker, cohort_mocker=mock_multidataset_cohort, stages=[DatasetStage1, SGStage1])
    print_job_by_stage()
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('SGStage1')

    assert len(get_multicohort().get_datasets()) == 2
    assert len(get_multicohort().get_sequencing_groups()) == 4
    assert get_batch().job_by_stage.get('DatasetStage1').get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('SGStage1').get('job_n') == len(get_multicohort().get_sequencing_groups())


def test_mixture_of_multiple_dataset_and_sg_stages(mocker: MockFixture, tmp_path):
    """
    Tests that when multiple SequencingGroupStages and DatasetStages are run together, the DatasetStage is run once per dataset and
    the SequencingGroupStage is run once per sequencing group.
    """

    set_config_to_default(tmp_path)

    from cpg_flow.inputs import get_multicohort
    from cpg_utils.hail_batch import get_batch

    run_workflow(
        mocker,
        cohort_mocker=mock_multidataset_cohort,
        stages=[
            DatasetStage1,
            DatasetStage2,
            SGStage1,
        ],
    )
    print_job_by_stage()
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('DatasetStage2')
    assert get_batch().job_by_stage.get('SGStage1')

    assert len(get_multicohort().get_datasets()) == 2
    assert len(get_multicohort().get_sequencing_groups()) == 4
    assert get_batch().job_by_stage.get('DatasetStage1').get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('DatasetStage2').get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('SGStage1').get('job_n') == len(get_multicohort().get_sequencing_groups())


def test_depcrecated_cohort_stage_implementation(mocker: MockFixture, tmp_path):
    """
    Tests the CohortStageImplementation, according to the deprecated behaviour (i.e. one cohort per run always)
    """

    set_config_to_default(tmp_path)

    from cpg_flow.inputs import get_multicohort
    from cpg_utils.hail_batch import get_batch

    run_workflow(mocker, cohort_mocker=mock_multidataset_cohort, stages=[CohortStage1])
    print_job_by_stage()
    assert get_batch().job_by_stage.get('CohortStage1')
    assert get_batch().job_by_stage.get('CohortStage1').get('job_n') == 1

    print(get_multicohort())


def test_mix_all_stage_types(mocker: MockFixture, tmp_path):
    """
    Test CohortStage, DatasetStage, and SequencingGroupStage together.
    """

    set_config_to_default(tmp_path)

    from cpg_flow.inputs import get_multicohort
    from cpg_utils.hail_batch import get_batch

    run_workflow(
        mocker,
        cohort_mocker=mock_multidataset_cohort,
        stages=[
            CohortStage1,
            DatasetStage1,
            DatasetStage2,
            SGStage1,
        ],
    )
    print_job_by_stage()
    assert get_batch().job_by_stage.get('CohortStage1')
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('DatasetStage2')
    assert get_batch().job_by_stage.get('SGStage1')

    assert len(get_multicohort().get_datasets()) == 2
    assert len(get_multicohort().get_sequencing_groups()) == 4

    assert get_batch().job_by_stage.get('CohortStage1').get('job_n') == 1
    assert get_batch().job_by_stage.get('DatasetStage1').get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('DatasetStage2').get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('SGStage1').get('job_n') == len(get_multicohort().get_sequencing_groups())


def test_multicohort_stage(mocker: MockFixture, tmp_path):
    """
    Test MultiCohortStage.
    """

    set_config_to_default(tmp_path)

    from cpg_utils.hail_batch import get_batch

    run_workflow(mocker, cohort_mocker=mock_multicohort, stages=[MultiCohortStage1])
    print_job_by_stage()
    assert get_batch().job_by_stage.get('MultiCohortStage1')
    assert get_batch().job_by_stage.get('MultiCohortStage1').get('job_n') == 1


def test_mixture_multicohort_and_other_stages(mocker: MockFixture, tmp_path):
    """
    Testing MultiChortStages with Cohort, Dataset and SequencingGroup stages.
    """

    set_config_to_default(tmp_path)

    from cpg_flow.inputs import MultiCohort, get_multicohort
    from cpg_utils.hail_batch import get_batch

    run_workflow(
        mocker,
        cohort_mocker=mock_multicohort,
        stages=[
            CohortStage1,
            DatasetStage1,
            DatasetStage2,
            SGStage1,
            MultiCohortStage1,
        ],
    )
    print_job_by_stage()
    multicohort = get_multicohort()
    assert isinstance(multicohort, MultiCohort)
    assert get_batch().job_by_stage.get('CohortStage1')
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('DatasetStage2')
    assert get_batch().job_by_stage.get('SGStage1')
    assert get_batch().job_by_stage.get('MultiCohortStage1')

    assert len(multicohort.get_datasets()) == 3
    assert len(multicohort.get_sequencing_groups()) == 6
    assert len(multicohort.get_cohorts()) == 2

    assert get_batch().job_by_stage.get('MultiCohortStage1').get('job_n') == 1
    assert get_batch().job_by_stage.get('DatasetStage1').get('job_n') == len(multicohort.get_datasets())
    assert get_batch().job_by_stage.get('DatasetStage2').get('job_n') == len(multicohort.get_datasets())
    assert get_batch().job_by_stage.get('SGStage1').get('job_n') == len(multicohort.get_sequencing_groups())
    assert get_batch().job_by_stage.get('CohortStage1').get('job_n') == len(multicohort.get_cohorts())

==> cpg_flow-0.2.5.tar.gz: cpg_flow-0.2.5/tests/stages/test_stage_types.py <==
"""
Test building stages DAG.
"""

from typing import Any, cast

from pytest_mock import MockFixture

from cpg_flow.inputs import MultiCohort, get_multicohort
from cpg_utils.hail_batch import get_batch

from tests import set_config
from tests.stages import (
    CohortStage1,
    DatasetStage1,
    DatasetStage2,
    MultiCohortStage1,
    SGStage1,
    mock_multicohort,
    mock_multidataset_cohort,
    run_workflow,
)


def set_config_to_default(tmp_path):
    set_config(get_config(tmp_path=tmp_path), tmp_path / 'config.toml')


def print_job_by_stage():
    print('Job by stage:', get_batch().job_by_stage)


def get_config(tmp_path) -> str:
    conf = f"""
    [workflow]
    access_level = 'test'
    dataset = 'fewgenomes'
    sequencing_type = 'genome'
    driver_image = 'stub'

    check_inputs = false
    check_intermediates = false

    # Skip stages with outputs that already exist
    check_expected_outputs = false

    [storage.default]
    default = '{tmp_path}'
    [storage.fewgenomes]
    default = '{tmp_path}'

    [hail]
    billing_project = 'fewgenomes'
    backend = 'local'
    dry_run = true
    """

    return conf


def get_config_multicohort(tmp_path) -> str:
    conf = f"""
    [workflow]
    access_level = 'test'
    dataset = 'fewgenomes'
    sequencing_type = 'genome'
    driver_image = 'stub'
    input_cohorts = ['COHA', 'COHB']

    check_inputs = false
    check_intermediates = false

    # Skip stages with outputs that already exist
    check_expected_outputs = false

    [storage.default]
    default = '{tmp_path}'
    [storage.fewgenomes]
    default = '{tmp_path}'

    [hail]
    billing_project = 'fewgenomes'
    backend = 'local'
    dry_run = true
    """

    return conf


def test_dataset_stages(mocker: MockFixture, tmp_path):
    """
    Test that dataset stages are run once per dataset.
    """
    set_config_to_default(tmp_path)

    run_workflow(mocker, cohort_mocker=mock_multidataset_cohort, stages=[DatasetStage1])
    print_job_by_stage()

    assert get_batch().job_by_stage.get('DatasetStage1')
    # The next two tests are looking at the same behaviour, but from different angles
    assert len(get_multicohort().get_datasets()) == 2
    assert get_batch().job_by_stage.get('DatasetStage1', {}).get('job_n') == len(get_multicohort().get_datasets())


def test_multiple_dataset_stages(mocker: MockFixture, tmp_path):
    """
    Test that dataset stages are run once per dataset.
    """

    set_config_to_default(tmp_path)

    run_workflow(mocker, cohort_mocker=mock_multidataset_cohort, stages=[DatasetStage1, DatasetStage2])
    print_job_by_stage()
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('DatasetStage2')

    assert len(get_multicohort().get_datasets()) == 2
    assert get_batch().job_by_stage.get('DatasetStage1', {}).get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('DatasetStage2', {}).get('job_n') == len(get_multicohort().get_datasets())


def test_mixture_dataset_stage_sg_stage(mocker: MockFixture, tmp_path):
    """
    Tests that when a SequencingGroupStage and DatasetStage are run together, the DatasetStage is run once per dataset and
    the SequencingGroupStage is run once per sequencing group.
    """

    set_config_to_default(tmp_path)

    run_workflow(mocker, cohort_mocker=mock_multidataset_cohort, stages=[DatasetStage1, SGStage1])
    print_job_by_stage()
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('SGStage1')

    assert len(get_multicohort().get_datasets()) == 2
    assert len(get_multicohort().get_sequencing_groups()) == 4
    assert get_batch().job_by_stage.get('DatasetStage1', {}).get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('SGStage1', {}).get('job_n') == len(get_multicohort().get_sequencing_groups())


def test_mixture_of_multiple_dataset_and_sg_stages(mocker: MockFixture, tmp_path):
    """
    Tests that when multiple SequencingGroupStages and DatasetStages are run together, the DatasetStage is run once per dataset and
    the SequencingGroupStage is run once per sequencing group.
    """

    set_config_to_default(tmp_path)

    run_workflow(
        mocker,
        cohort_mocker=mock_multidataset_cohort,
        stages=[
            DatasetStage1,
            DatasetStage2,
            SGStage1,
        ],
    )
    print_job_by_stage()
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('DatasetStage2')
    assert get_batch().job_by_stage.get('SGStage1')

    assert len(get_multicohort().get_datasets()) == 2
    assert len(get_multicohort().get_sequencing_groups()) == 4
    assert get_batch().job_by_stage.get('DatasetStage1', {}).get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('DatasetStage2', {}).get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('SGStage1', {}).get('job_n') == len(get_multicohort().get_sequencing_groups())


def test_depcrecated_cohort_stage_implementation(mocker: MockFixture, tmp_path):
    """
    Tests the CohortStageImplementation, according to the deprecated behaviour (i.e. one cohort per run always)
    """

    set_config_to_default(tmp_path)

    run_workflow(mocker, cohort_mocker=mock_multidataset_cohort, stages=[CohortStage1])
    print_job_by_stage()
    assert get_batch().job_by_stage.get('CohortStage1')
    assert get_batch().job_by_stage.get('CohortStage1', {}).get('job_n') == 1

    print(get_multicohort())


def test_mix_all_stage_types(mocker: MockFixture, tmp_path):
    """
    Test CohortStage, DatasetStage, and SequencingGroupStage together.
    """

    set_config_to_default(tmp_path)

    run_workflow(
        mocker,
        cohort_mocker=mock_multidataset_cohort,
        stages=[
            CohortStage1,
            DatasetStage1,
            DatasetStage2,
            SGStage1,
        ],
    )
    print_job_by_stage()
    assert get_batch().job_by_stage.get('CohortStage1')
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('DatasetStage2')
    assert get_batch().job_by_stage.get('SGStage1')

    assert len(get_multicohort().get_datasets()) == 2
    assert len(get_multicohort().get_sequencing_groups()) == 4

    assert get_batch().job_by_stage.get('CohortStage1', {}).get('job_n') == 1
    assert get_batch().job_by_stage.get('DatasetStage1', {}).get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('DatasetStage2', {}).get('job_n') == len(get_multicohort().get_datasets())
    assert get_batch().job_by_stage.get('SGStage1', {}).get('job_n') == len(get_multicohort().get_sequencing_groups())


def test_multicohort_stage(mocker: MockFixture, tmp_path):
    """
    Test MultiCohortStage.
    """

    set_config_to_default(tmp_path)

    run_workflow(mocker, cohort_mocker=mock_multicohort, stages=[MultiCohortStage1])
    print_job_by_stage()
    assert get_batch().job_by_stage.get('MultiCohortStage1')
    assert get_batch().job_by_stage.get('MultiCohortStage1', {}).get('job_n') == 1


def test_mixture_multicohort_and_other_stages(mocker: MockFixture, tmp_path):
    """
    Testing MultiChortStages with Cohort, Dataset and SequencingGroup stages.
    """

    set_config_to_default(tmp_path)

    run_workflow(
        mocker,
        cohort_mocker=mock_multicohort,
        stages=[
            CohortStage1,
            DatasetStage1,
            DatasetStage2,
            SGStage1,
            MultiCohortStage1,
        ],
    )
    print_job_by_stage()
    multicohort = get_multicohort()
    assert isinstance(multicohort, MultiCohort)
    assert get_batch().job_by_stage.get('CohortStage1')
    assert get_batch().job_by_stage.get('DatasetStage1')
    assert get_batch().job_by_stage.get('DatasetStage2')
    assert get_batch().job_by_stage.get('SGStage1')
    assert get_batch().job_by_stage.get('MultiCohortStage1')

    assert len(multicohort.get_datasets()) == 3
    assert len(multicohort.get_sequencing_groups()) == 6
    assert len(multicohort.get_cohorts()) == 2

    assert get_batch().job_by_stage.get('MultiCohortStage1', {}).get('job_n') == 1
    assert get_batch().job_by_stage.get('DatasetStage1', {}).get('job_n') == len(multicohort.get_datasets())
    assert get_batch().job_by_stage.get('DatasetStage2', {}).get('job_n') == len(multicohort.get_datasets())
    assert get_batch().job_by_stage.get('SGStage1', {}).get('job_n') == len(multicohort.get_sequencing_groups())
    assert get_batch().job_by_stage.get('CohortStage1', {}).get('job_n') == len(multicohort.get_cohorts())
